package gov.usgs.earthquake.nshmp.netcdf.www;

import java.io.IOException;
import java.nio.file.Path;

import gov.usgs.earthquake.nshmp.netcdf.NetcdfGroundMotions;
import gov.usgs.earthquake.nshmp.www.NshmpMicronautServlet;

import io.micronaut.context.annotation.Value;
import io.micronaut.context.event.StartupEvent;
import io.micronaut.http.HttpRequest;
import io.micronaut.http.HttpResponse;
import io.micronaut.http.MediaType;
import io.micronaut.http.annotation.Controller;
import io.micronaut.http.annotation.Get;
import io.micronaut.runtime.event.annotation.EventListener;
import io.swagger.v3.core.util.Yaml;
import io.swagger.v3.oas.annotations.Hidden;
import io.swagger.v3.oas.annotations.tags.Tag;
import jakarta.inject.Inject;

/**
 * Expose OpenAPI YAML file.
 *
 * @author U.S. Geological Survey
 */
@Tag(
    name = "Swagger",
    description = "Swagger OpenAPI YAML")
@Hidden
@Controller("/swagger")
public class SwaggerController {

  @Inject
  private NshmpMicronautServlet servlet;

  @Value("${nshmp-ws-static.netcdf-file}")
  Path netcdfPath;

  NetcdfGroundMotions netcdf;

  /**
   * Read in data type and return the appropriate service to use.
   */
  @EventListener
  void startup(StartupEvent event) {
    netcdf = new NetcdfGroundMotions(netcdfPath);
  }

  @Get(produces = MediaType.TEXT_EVENT_STREAM)
  public HttpResponse<String> doGet(HttpRequest<?> request) {
    try {
      var openApi = NetcdfWsUtils.getOpenApi(
          request,
          netcdf.netcdfData(),
          servicePatternSection(request));
      return HttpResponse.ok(Yaml.pretty(openApi));
    } catch (Exception e) {
      return NetcdfWsUtils.handleError(e, "Swagger", request.getUri().getPath());
    }
  }

  private static String servicePatternSection(HttpRequest<?> request)
      throws IOException {
    var url = NetcdfWsUtils.getRequestUrl(request);
    url = url.endsWith("/swagger") ? url.replace("/swagger", "") : url;

    return new StringBuilder()
        .append(
            "<details>\n" +
                "<summary>Service Call Patterns</summary>\n")
        .append(
            "### Query Pattern\n" +

                "The query based service call is in the form of:\n" +

                "```text\n" +
                url + "/spectra?latitude={number}&longitude={number}\n" +
                url + "/spectra?latitude={number}&longitude={number}&siteClass={string}\n" +
                "````\n" +

                "Example:\n" +
                "```text\n" +
                url + "/spectra?latitude=34&longitude=-118\n" +
                url + "/spectra?latitude=34&longitude=-118&siteClass=BC\n" +
                "```\n")
        .append(
            "### Slash Pattern\n" +

                "The slash based service call is in the form of:\n" +

                "```text\n" +
                url + "/spectra/{latitude}/{longitude}\n" +
                url + "/spectra/{latitude}/{longitude}/{siteClass}\n" +
                "```\n" +

                "Example:\n" +

                "```text\n" +
                url + "/spectra/34/-118\n" +
                url + "/spectra/34/-118/BC\n" +
                "```\n")
        .append("</details>")
        .toString();
  }

}
