package gov.usgs.earthquake.nshmp.netcdf.www;

import java.nio.file.Path;
import java.util.List;

import gov.usgs.earthquake.nshmp.gmm.Imt;
import gov.usgs.earthquake.nshmp.gmm.NehrpSiteClass;
import gov.usgs.earthquake.nshmp.netcdf.NetcdfHazardCurves;
import gov.usgs.earthquake.nshmp.netcdf.www.NetcdfService.RequestDataImt;
import gov.usgs.earthquake.nshmp.netcdf.www.NetcdfService.RequestDataSiteClass;
import gov.usgs.earthquake.nshmp.netcdf.www.NetcdfService.ResponseData;
import gov.usgs.earthquake.nshmp.netcdf.www.NetcdfServiceHazardCurves.HazardResponseMetadata;
import gov.usgs.earthquake.nshmp.www.NshmpMicronautServlet;
import gov.usgs.earthquake.nshmp.www.ResponseBody;

import io.micronaut.context.annotation.Value;
import io.micronaut.context.event.StartupEvent;
import io.micronaut.core.annotation.Nullable;
import io.micronaut.http.HttpRequest;
import io.micronaut.http.HttpResponse;
import io.micronaut.http.MediaType;
import io.micronaut.http.annotation.Controller;
import io.micronaut.http.annotation.Get;
import io.micronaut.http.annotation.PathVariable;
import io.micronaut.http.annotation.QueryValue;
import io.micronaut.runtime.event.annotation.EventListener;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.media.Content;
import io.swagger.v3.oas.annotations.media.Schema;
import io.swagger.v3.oas.annotations.responses.ApiResponse;
import io.swagger.v3.oas.annotations.tags.Tag;
import jakarta.inject.Inject;

/**
 * Micronaut controller for getting static hazards or ground motions from a
 * NetCDF file.
 *
 * @see NetcdfService
 *
 * @author U.S. Geological Survey
 */
@Tag(name = "Static Hazard Data")
@Controller("/hazard")
public class NetcdfController {

  @Inject
  private NshmpMicronautServlet servlet;

  @Value("${nshmp-ws-static.netcdf-file}")
  Path netcdfPath;

  NetcdfServiceHazardCurves service;

  /**
   * Read in data type and return the appropriate service to use.
   */
  @EventListener
  void startup(StartupEvent event) {
    var netcdfHazard = new NetcdfHazardCurves(netcdfPath);
    service = new NetcdfServiceHazardCurves(netcdfHazard);
  }

  /**
   * GET method to return static curves using slash delimited.
   *
   * @param request The HTTP request
   * @param longitude The longitude of the site
   * @param latitude Latitude of the site
   * @param siteClass The site class
   */
  @Operation(
      summary = "Returns static curves given a longitude, latitude, site class, and IMT",
      description = "### Returns hazard curves for a " +
          "user-specified latitude, longitude, site class, and IMT.\n" +

          "Enter the latitude and longitude select the site class and IMT, and press `Execute`.\n" +

          "### Service call pattern\n" +
          "This service call is slashed delimited with pattern: " +
          "`/hazard/{longitude}/{latitude}/{siteClass}/{imt}`\n" +
          "<br><br>" +
          "Example: `/hazard/-118/34/BC/PGA`",
      operationId = "hazard-by-imt")
  @ApiResponse(
      description = "Spatially interpolated hazard curves",
      responseCode = "200",
      content = @Content(
          schema = @Schema(implementation = ResponseByImt.class)))
  @Get(uri = "/{longitude}/{latitude}/{siteClass}/{imt}", produces = MediaType.APPLICATION_JSON)
  public HttpResponse<String> doGetSlashByImt(
      HttpRequest<?> request,
      @Schema(required = true) @PathVariable @Nullable Double longitude,
      @Schema(required = true) @PathVariable @Nullable Double latitude,
      @Schema(required = true) @PathVariable @Nullable NehrpSiteClass siteClass,
      @Schema(required = true) @PathVariable @Nullable Imt imt) {
    return doGet(request, longitude, latitude, siteClass, imt);
  }

  /**
   * GET method to return static curves using slash delimited.
   *
   * @param request The HTTP request
   * @param longitude The longitude of the site
   * @param latitude Latitude of the site
   * @param siteClass The site class
   */
  @Operation(
      summary = "Returns static curves given a longitude, latitude, and site class.",
      description = "### Returns hazard curves for a " +
          "user-specified latitude, longitude, and site class.\n" +

          "Enter the latitude and longitude select the site class, and press `Execute`.\n" +

          "### Service call pattern\n" +
          "This service call is slashed delimited with pattern: " +
          "`/hazard/{longitude}/{latitude}/{siteClass}`\n" +
          "<br><br>" +
          "Example: `/hazard/-118/34/BC`",
      operationId = "hazard-by-siteclass")
  @ApiResponse(
      description = "Spatially interpolated hazard curves",
      responseCode = "200",
      content = @Content(
          schema = @Schema(implementation = ResponseBySiteClass.class)))
  @Get(uri = "/{longitude}/{latitude}/{siteClass}", produces = MediaType.APPLICATION_JSON)
  public HttpResponse<String> doGetSlashBySite(
      HttpRequest<?> request,
      @Schema(required = true) @PathVariable @Nullable Double longitude,
      @Schema(required = true) @PathVariable @Nullable Double latitude,
      @Schema(required = true) @PathVariable @Nullable NehrpSiteClass siteClass) {
    return doGet(request, longitude, latitude, siteClass, null);
  }

  /**
   * GET method to return hazard curves using slash delimited.
   *
   * @param request The HTTP request
   * @param longitude The longitude of the site
   * @param latitude Latitude of the site
   */
  @Operation(
      summary = "Returns static curves given a longitude and latitude",
      description = "### Returns hazard curves for all site class for a " +
          "user-specified latitude and longitude.\n" +

          "Enter the latitude and longitude and press `Execute`.\n" +

          "### Service call pattern\n" +
          "This service call is slashed delimited with pattern: " +
          "`/hazard/{longitude}/{latitude}`\n" +
          "<br><br>" +
          "Example: `/hazard/-118/34`",
      operationId = "hazard")
  @ApiResponse(
      description = "Returns static curves from the NSHM NetCDF file",
      responseCode = "200",
      content = @Content(
          schema = @Schema(implementation = Response.class)))
  @Get(uri = "/{longitude}/{latitude}", produces = MediaType.APPLICATION_JSON)
  public HttpResponse<String> doGetSlash(
      HttpRequest<?> request,
      @Schema(required = true) @PathVariable @Nullable Double longitude,
      @Schema(required = true) @PathVariable @Nullable Double latitude) {
    return doGet(request, longitude, latitude, null, null);
  }

  /**
   * GET method to return a static curve using URL query.
   *
   * @param request The HTTP request
   * @param longitude The longitude of the site
   * @param latitude Latitude of the site
   * @param siteClass The site class (optional)
   */
  @Operation(
      summary = "Returns static curves given a longitude, latitude, site class, and IMT",
      description = "### Returns hazard curves for a " +
          "user-specified latitude, longitude, site class, and IMT.\n" +

          "Enter the latitude and longitude select the site class (optional) and " +
          "IMT (optional), and press `Execute`.\n" +

          "### Service call pattern\n" +
          "This service call is query based with pattern: " +
          "`/hazard?longitude={number}&latitude={number}&siteClass={string}&imt={string}`\n" +
          "<br><br>" +
          "Example: `/hazard?longitude=-118&latitude=34&siteClass=A&imt=PGA`",
      operationId = "hazard-by-imt")
  @ApiResponse(
      description = "Spatially interpolated hazard curves",
      responseCode = "200",
      content = @Content(
          schema = @Schema(implementation = ResponseByImt.class)))
  @Get(uri = "{?longitude,latitude,siteClass,imt}", produces = MediaType.APPLICATION_JSON)
  public HttpResponse<String> doGet(
      HttpRequest<?> request,
      @Schema(required = true) @QueryValue @Nullable Double longitude,
      @Schema(required = true) @QueryValue @Nullable Double latitude,
      @QueryValue @Nullable NehrpSiteClass siteClass,
      @QueryValue @Nullable Imt imt) {
    var query = new HazardQuery(longitude, latitude, siteClass, imt);
    return service.handleServiceCall(request, query);
  }

  // For Swagger schema
  private static class ResponseByImt
      extends
      ResponseBody<RequestDataImt, ResponseData<HazardResponseMetadata>> {}

  // For Swagger schema
  private static class ResponseBySiteClass
      extends
      ResponseBody<RequestDataSiteClass, List<ResponseData<HazardResponseMetadata>>> {}

  // For Swagger schema
  private static class Response
      extends
      ResponseBody<RequestDataSiteClass, List<List<ResponseData<HazardResponseMetadata>>>> {}
}
